﻿// Copyright 2009-2010 Oliver Sturm <oliver@sturmnet.org> All rights reserved. 

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Dynamic;

namespace Basics {
  class Program {
    static void Main(string[] args) {
      dynamic mc = new MyClass();
      mc.DoSomething();

      Console.WriteLine(mc * 2);

      dynamic expando = new ExpandableObject( );
      expando.Name = "Oliver Sturm";
      expando.Age = 23;
      Console.WriteLine("{0} is {1}", expando.Name, expando.Age);
    }
  }

  public class MyClass: DynamicObject {
    public MyClass( ) {
      
    }

    public override bool TryBinaryOperation(BinaryOperationBinder binder, object arg, out object result) {
      Console.WriteLine("TryBinaryOperation");
      result = 42;
      return true;
    }

    public override IEnumerable<string> GetDynamicMemberNames( ) {
      Console.WriteLine("GetDynamicMemberNames");
      return base.GetDynamicMemberNames( );
    }

    public override bool TryGetMember(GetMemberBinder binder, out object result) {
      Console.WriteLine("TryGetMember");
      return base.TryGetMember(binder, out result);
    }

    public override bool TryInvoke(InvokeBinder binder, object[] args, out object result) {
      Console.WriteLine("TryInvoke");
      return base.TryInvoke(binder, args, out result);
    }

    public override bool TryInvokeMember(InvokeMemberBinder binder, object[] args, out object result) {
      result = null;

      if (binder.Name == "DoSomething") {
        Console.WriteLine("Now doing something!");
        return true;
      }

      return false;
    }
  }

  public class ExpandableObject : DynamicObject {
    public ExpandableObject( ) {
      dict = new Dictionary<string, object>( );
    }

    Dictionary<string, object> dict;

    public override bool TryGetMember(GetMemberBinder binder, out object result) {
      if (dict.ContainsKey(binder.Name)) {
        result = dict[binder.Name];
        return true;
      }
      else {
        result = null;
        return false;
      }
    }

    public override bool TrySetMember(SetMemberBinder binder, object value) {
      dict[binder.Name] = value;
      return true;
    }
  }
}
