// Copyright 2009-2010 Oliver Sturm <oliver@sturmnet.org> All rights reserved. 

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Dynamic;
using System.Data;
using System.Data.OleDb;

namespace DynamicDB {
  public class DynamicTable : DynamicObject, IEnumerable<DynamicDataRow> {
    public DynamicTable(DynamicDatabase db, string name)
      : base( ) {
      this.db = db;
      this.name = name;
    }

    DynamicDatabase db;
    string name;
    public string Name {
      get {
        return name;
      }
    }
    bool filled;
    DataTable table;
    internal DataTable DataTable {
      get {
        if (!filled)
          FillData( );
        return table;
      }
    }

    void FillData( ) {
      table = new DataTable(name);
      OleDbDataAdapter adapter =
        new OleDbDataAdapter("select * from " + name, db.Connection);
      adapter.Fill(table);
      filled = true;
    }

    public override bool TryGetMember(GetMemberBinder binder, out object result) {
      if (binder.Name == "Fields") {
        result = new DynamicFields(db, this);
        return true;
      }

      return base.TryGetMember(binder, out result);
    }

    #region IEnumerable<DynamicDataRow> Members

    IEnumerator<DynamicDataRow> IEnumerable<DynamicDataRow>.GetEnumerator( ) {
      if (!filled)
        FillData( );

      foreach (DataRow row in table.Rows)
        yield return new DynamicDataRow(row);
    }

    #endregion

    #region IEnumerable Members

    System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator( ) {
      return ((IEnumerable<DynamicDataRow>) this).GetEnumerator( );
    }

    #endregion
  }
}